@@ -12,6 +12,7 @@ from functools import partial
 from .. import errors
 from .. import util
 
+from . common import CLOSED
 from . import file
 from . import http
 from . import nbd
@@ -48,6 +49,27 @@ class Closer:
         self.close = func
 
 
+class Wrapper:
+    """
+    Used to lend a backend without closing the wrapped backend.
+    """
+
+    def __init__(self, backend):
+        self._backend = backend
+
+    def close(self):
+        self._backend = CLOSED
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, t, v, tb):
+        self.close()
+
+    def __getattr__(self, name):
+        return getattr(self._backend, name)
+
+
 def supports(name):
     return name in _modules
 
@@ -98,6 +120,9 @@ def get(req, ticket, config):
             ctx.close()
             raise
 
+        # Authorized connections get a longer timeout from the ticket.
+        req.set_connection_timeout(ticket.inactivity_timeout)
+
         # Register a closer removing the context when the connection is closed.
         req.context[ticket.uuid] = Closer(
             partial(ticket.remove_context, req.connection_id))
