@@ -89,6 +89,7 @@ import sys
 
 from collections import namedtuple
 
+from . import nbd
 from . import util
 
 # NBD spec allows zeroing up to 2**32 - 1 bytes, buf some nbd servers like
@@ -115,8 +116,6 @@ def extents(client, offset=0, length=None, dirty=False):
     else:
         end = offset + length
 
-    meta_context = client.dirty_bitmap if dirty else "base:allocation"
-
     # NBD limit extents request to 4 GiB - 1. We use smaller step to limit the
     # number of extents kept in memory when accessing very fragmented images.
     max_step = 2 * 1024**3
@@ -131,7 +130,16 @@ def extents(client, offset=0, length=None, dirty=False):
         step = min(end - offset, max_step)
         res = client.extents(offset, step)
 
-        for ext in res[meta_context]:
+        if dirty:
+            extents = merged(
+                res[nbd.BASE_ALLOCATION], res[client.dirty_bitmap])
+        elif nbd.QEMU_ALLOCATION_DEPTH in res:
+            extents = merged(
+                res[nbd.BASE_ALLOCATION], res[nbd.QEMU_ALLOCATION_DEPTH])
+        else:
+            extents = res[nbd.BASE_ALLOCATION]
+
+        for ext in extents:
             # Handle the case of last extent of the last block status command
             # exceeding requested range.
             if offset + ext.length > end:
@@ -156,6 +164,46 @@ def extents(client, offset=0, length=None, dirty=False):
     yield cur
 
 
+def merged(extents_a, extents_b):
+    """
+    Merge lists of extents with distinct flags bits, yielding merged extents
+    with flags from both lists. Yielding ends when the first iterator is
+    consumed.
+
+    Yields nbd.Extent() including all bits from both extents.
+    """
+    iter_a = iter(extents_a)
+    iter_b = iter(extents_b)
+
+    a = None
+    b = None
+
+    while True:
+        try:
+            if a is None:
+                a = next(iter_a)
+            if b is None:
+                b = next(iter_b)
+        except StopIteration:
+            return
+
+        if a.length == b.length:
+            # The easy case, merge and clear both.
+            yield nbd.Extent(a.length, a.flags | b.flags)
+            a = None
+            b = None
+        elif a.length > b.length:
+            # Yield the overlapping area and keep rest of a.
+            yield nbd.Extent(b.length, a.flags | b.flags)
+            a = nbd.Extent(a.length - b.length, a.flags)
+            b = None
+        else:  # a.length < b.length
+            # Yield the overlapping area and keep rest of b.
+            yield nbd.Extent(a.length, a.flags | b.flags)
+            b = nbd.Extent(b.length - a.length, b.flags)
+            a = None
+
+
 def copy(src_client, dst_client, block_size=4 * 1024**2, queue_depth=4,
          progress=None):
     """
